/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * Simple particle system to demonstrate the use of a class to instantiate 
 * numerous objects ('Particle' class). Attract and repel particles on 
 * right/left mouse button presses. 
 * Developed at Processing Course 2018 AdBK Nuremberg
 *****************************************************************************/


int numParticles = 1000; // number of particles
Particle[] particles;    // array containing all particles

void setup() {
  size(1000, 800);
  
  // create particle array
  particles = new Particle[numParticles];
  
  for (int i = 0; i < particles.length; i++) {
    
    // compute random position
    PVector mitte = new PVector(random(20, width-20), random(20, height-20));
    
    // instantiate new particle object with random position and radius in range 8 to 20
    // and yellow color
    particles[i] = new Particle(mitte,
                                random(8, 20),
                                color(255, 255, 0, 100)
                               );
  }
  
  // set color of first particle to red, of last particle to green (for debugging)
  particles[0].setColor(color(255, 0, 0, 100));
  particles[particles.length-1].setColor(color(0, 255, 0, 100));
  
  // specify circles by center and radius
  ellipseMode(RADIUS);
  
}

void draw() {
  background(0);
  
  // if right/left mouse button is pressed attract/repel particles to/from mouse position
  if (mousePressed == true) {
    
    // mouse position
    PVector mouse = new PVector(mouseX, mouseY);
  
    if (mouseButton == LEFT) {
      
      for (int i = 0; i < particles.length; i++) {  
        float d = PVector.dist(particles[i].getM(), mouse); // compute distance of mouse and current particle
        PVector force = PVector.sub(particles[i].getM(), mouse).mult(3.0/d); // compute force vector and attenuate by distance
        particles[i].addForce(force); // add force vector to particle
      }   
    }
    else if (mouseButton == RIGHT) {
      
      for (int i = 0; i < particles.length; i++) {  
        float d = PVector.dist(particles[i].getM(), mouse); // compute distance of mouse and current particle
        PVector force = PVector.sub(mouse, particles[i].getM()).mult(3.0/d); // compute force vector and attenuate by distance
        particles[i].addForce(force); // add force vector to particle
      }   
    }
    
  }
  
  // update and draw particles
  for (int i = 0; i < particles.length; i++) {  
    particles[i].update();
    particles[i].display();   
  }
  
}
