/*
 * Software License Agreement (BSD License)
 *
 * Copyright (c) 2018 Oliver Mayer, Akademie der Bildenden Kuenste Nuernberg. 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * - Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*****************************************************************************  
 * The particle class
 *****************************************************************************/


// Particle class 
class Particle {
  
  PVector m; // particle position (mittelpunkt)
  PVector v; // particle velocity
  
  float r; // particle radius
  color c; // particle color
  
  
  
  // Standard Constructor
  Particle() {
    c = color(255, 0, 0, 128);
    r = 10;
    m = new PVector(20, 20);
    v = new PVector();
  }
  
  // Constructor with initial values for position (mittelpunkt), radius and color
  Particle(PVector mittelpunkt, float radius, color col) {
    m = mittelpunkt;
    r = radius;
    c = col;
    v = new PVector();
  }
  
  // set new particle position
  private void setM(PVector neuerMittelpunkt) {
    m = neuerMittelpunkt; 
  }
  
  // get particle position
  PVector getM() {
    return m; 
  }
  
  // set new particle color
  void setColor(color col) {
    c = col; 
  }
  
  // add force to particle. the vector 'force' is added to the particle's current velocity 
  void addForce(PVector force) {
     v.add(force);
  }
  
  // update particle position. the particle's current position is computed using its current velocity
  void update() {
    
    // add velocity vector to position
    m.add(v);
    
    // attenuate velocity depending on particle's size
    // (a large particle's velocity is reduced more than a small particle's velocity ("Reibung"))
    float rd = map(r, 8, 25, 0, 1);
    
    v.x = v.x*0.8/(rd+0.7);
    v.y = v.y*0.8/(rd+0.7);
  }
  
  // draw particle
  void display() {
    
    noStroke();
    fill(c);
    ellipse(m.x, m.y, r, r);
  }
  
  
  
  
  
}
